@@ -314,7 +314,8 @@ sub _handle_one_result {
         $row->{server}       = $servhref->{servername};
         $row->{breedingid}   = $breedingid;
         $row->{isbn}=_isbn_replace($row->{isbn});
-        $row = _add_custom_field_rowdata($row, $marcrecord);
+        my $pref_newtags = C4::Context->preference('AdditionalFieldsInZ3950ResultSearch');
+        $row = _add_custom_field_rowdata( $row, $marcrecord, $pref_newtags );
     }
     return ( $row, $error );
 }
@@ -345,8 +346,8 @@ sub _do_xslt_proc {
 
 sub _add_custom_field_rowdata
 {
-    my ( $row, $record ) = @_;
-    my $pref_newtags = C4::Context->preference('AdditionalFieldsInZ3950ResultSearch');
+    my ( $row, $record, $pref_newtags ) = @_;
+    $pref_newtags //= '';
     my $pref_flavour = C4::Context->preference('MarcFlavour');
 
     $pref_newtags =~ s/^\s+|\s+$//g;
@@ -602,6 +603,10 @@ sub Z3950SearchAuth {
                             $row_data{heading}      = $heading;
                             $row_data{authid}       = $authid;
                             $row_data{heading_code} = $heading_authtype_code;
+                            $row_data{row}          = _add_custom_field_rowdata(
+                                { %row_data }, $marcrecord,
+                                C4::Context->preference('AdditionalFieldsInZ3950ResultAuthSearch')
+                            ) if C4::Context->preference('AdditionalFieldsInZ3950ResultAuthSearch');
                             push( @breeding_loop, \%row_data );
                         }
                         else {
