@@ -347,7 +347,7 @@ sub getFacets {
             },
             {
                 idx => 'ccode',
-                label => 'CollectionCodes',
+                label => 'Collections',
                 tags => [ qw / 099t 955h / ],
             }
             ];
@@ -362,7 +362,7 @@ sub getFacets {
                         @$facets,
                         {
                             idx   => 'holdingbranch',
-                            label => 'HoldingLibrary',
+                            label => 'Holding libraries',
                             tags  => [qw / 995c /],
                         }
                     );
@@ -375,7 +375,7 @@ sub getFacets {
                     @$facets,
                     {
                         idx   => 'homebranch',
-                        label => 'HomeLibrary',
+                        label => 'Home libraries',
                         tags  => [qw / 995b /],
                     }
                 );
@@ -422,7 +422,7 @@ sub getFacets {
             },
             {
                 idx   => 'itype',
-                label => 'ItemTypes',
+                label => 'Item types',
                 tags  => [ qw/ 952y 942c / ],
                 sep   => ', ',
             },
@@ -433,7 +433,7 @@ sub getFacets {
             },
             {
                 idx => 'ccode',
-                label => 'CollectionCodes',
+                label => 'Collections',
                 tags => [ qw / 9528 / ],
             }
             ];
@@ -448,7 +448,7 @@ sub getFacets {
                         @$facets,
                         {
                             idx   => 'holdingbranch',
-                            label => 'HoldingLibrary',
+                            label => 'Holding libraries',
                             tags  => [qw / 952b /],
                         }
                     );
@@ -461,7 +461,7 @@ sub getFacets {
                     @$facets,
                     {
                         idx   => 'homebranch',
-                        label => 'HomeLibrary',
+                        label => 'Home libraries',
                         tags  => [qw / 952a /],
                     }
                 );
@@ -581,6 +581,8 @@ sub display_marc_indicators {
 sub GetNormalizedUPC {
     my ($marcrecord,$marcflavour) = @_;
 
+    $marcflavour ||= C4::Context->preference('marcflavour');
+
     return unless $marcrecord;
     if ($marcflavour eq 'UNIMARC') {
         my @fields = $marcrecord->field('072');
@@ -597,7 +599,7 @@ sub GetNormalizedUPC {
         foreach my $field (@fields) {
             my $indicator = $field->indicator(1);
             my $upc = _normalize_match_point($field->subfield('a'));
-            if ($upc && $indicator == 1 ) {
+            if ($upc && $indicator eq '1' ) {
                 return $upc;
             }
         }
@@ -656,7 +658,7 @@ sub GetNormalizedEAN {
         foreach my $field (@fields) {
             my $indicator = $field->indicator(1);
             my $ean = _normalize_match_point($field->subfield('a'));
-            if ( $ean && $indicator == 3  ) {
+            if ( $ean && $indicator eq '3'  ) {
                 return $ean;
             }
         }
@@ -667,6 +669,8 @@ sub GetNormalizedOCLCNumber {
     my ($marcrecord,$marcflavour) = @_;
     return unless $marcrecord;
 
+    $marcflavour ||= C4::Context->preference('marcflavour');
+
     if ($marcflavour ne 'UNIMARC' ) {
         my @fields = $marcrecord->field('035');
         foreach my $field (@fields) {
@@ -679,7 +683,6 @@ sub GetNormalizedOCLCNumber {
     } else {
         # TODO for UNIMARC
     }
-    return
 }
 
 sub _normalize_match_point {
