@@ -35,8 +35,7 @@ use Koha::Email;
 use Koha::Exceptions;
 use Koha::Notice::Messages;
 use Koha::Notice::Templates;
-use Koha::DateUtils qw( dt_from_string output_pref );
-use Koha::Auth::TwoFactorAuth;
+use Koha::Notice::Util;
 use Koha::Patrons;
 use Koha::SMS::Providers;
 use Koha::SMTP::Servers;
@@ -67,6 +66,8 @@ BEGIN {
     );
 }
 
+our $domain_limits = {};
+
 =head1 NAME
 
 C4::Letters - Give functions for Letters management
@@ -329,11 +330,12 @@ sub SendAlerts {
             # FIXME: This 'default' behaviour should be moved to Koha::Email
             my $mail = Koha::Email->create(
                 {
-                    to       => $email,
-                    from     => $library->branchemail,
-                    reply_to => $library->branchreplyto,
-                    sender   => $library->branchreturnpath,
-                    subject  => "" . $letter->{title},
+                    to          => $email,
+                    from        => $library->branchemail,
+                    reply_to    => $library->branchreplyto,
+                    sender      => $library->branchreturnpath,
+                    subject     => "" . $letter->{title},
+                    template_id => $letter->{id},
                 }
             );
 
@@ -485,16 +487,31 @@ sub SendAlerts {
                 cc => join( ',', @cc ),
                 (
                     (
-                        C4::Context->preference("ClaimsBccCopy")
-                          && ( $type eq 'claimacquisition'
+                        C4::Context->preference("ClaimsBccCopy") && ( $type eq 'claimacquisition'
                             || $type eq 'claimissues' )
                     )
                     ? ( bcc => $userenv->{emailaddress} )
                     : ()
                 ),
-                from => $library->branchemail
-                  || C4::Context->preference('KohaAdminEmailAddress'),
-                subject => "" . $letter->{title},
+                from => (
+                    $type eq 'claimissues'
+                    ? C4::Context->preference('SerialsDefaultEmailAddress')
+                    : C4::Context->preference('AcquisitionsDefaultEmailAddress')
+                    )
+                    || $library->branchemail
+                    || C4::Context->preference('KohaAdminEmailAddress'),
+                (
+                    $type eq 'claimissues' && C4::Context->preference('SerialsDefaultReplyTo')
+                    ? ( reply_to => C4::Context->preference('SerialsDefaultReplyTo') )
+                    : (
+                        ( $type eq 'claimacquisition' || $type eq 'orderacquisition' )
+                            && C4::Context->preference('AcquisitionsDefaultReplyTo')
+                        ? ( reply_to => C4::Context->preference('AcquisitionsDefaultReplyTo') )
+                        : ()
+                    )
+                ),
+                subject     => "" . $letter->{title},
+                template_id => $letter->{id},
             }
         );
 
@@ -597,6 +614,7 @@ sub GetPreparedLetter {
         $letter = $template->unblessed;
         $letter->{'content-type'} = 'text/html; charset="UTF-8"' if $letter->{is_html};
     }
+    $lang = $letter->{'lang'};
 
     my $objects = $params{objects} || {};
     my $tables = $params{tables} || {};
@@ -608,6 +626,14 @@ sub GetPreparedLetter {
          return;
     my $want_librarian = $params{want_librarian};
 
+    # Best guess at language 'default' notice is written for include handling
+    if ( $lang eq 'default' ) {
+
+        # Pick the first selected syspref language
+        my @languages = split /,/, C4::Context->preference('OPACLanguages');
+        $lang = shift @languages;
+    }
+
     $letter->{content} = _process_tt(
         {
             content    => $letter->{content},
@@ -702,6 +728,8 @@ sub GetPreparedLetter {
 
     $letter->{content} =~ s/<<\S*>>//go; #remove any stragglers
 
+    Koha::Plugins->call( 'transform_prepared_letter', { letter => $letter, params => \%params } );
+
     return $letter;
 }
 
@@ -747,6 +775,8 @@ sub _parseletter_sth {
     ($table eq 'accountlines' )    ? "SELECT * FROM $table WHERE   accountlines_id = ?"                               :
     ($table eq 'biblio'       )    ? "SELECT * FROM $table WHERE   biblionumber = ?"                                  :
     ($table eq 'biblioitems'  )    ? "SELECT * FROM $table WHERE   biblionumber = ?"                                  :
+    ($table eq 'tickets'      )    ? "SELECT * FROM $table WHERE   id = ?"                                            :
+    ($table eq 'ticket_updates' )  ? "SELECT * FROM $table WHERE   id = ?"                                            :
     ($table eq 'credits'      )    ? "SELECT * FROM accountlines WHERE   accountlines_id = ?"                         :
     ($table eq 'debits'       )    ? "SELECT * FROM accountlines WHERE   accountlines_id = ?"                         :
     ($table eq 'items'        )    ? "SELECT * FROM $table WHERE     itemnumber = ?"                                  :
@@ -765,7 +795,7 @@ sub _parseletter_sth {
     ($table eq 'subscription') ? "SELECT * FROM $table WHERE subscriptionid = ?" :
     ($table eq 'serial') ? "SELECT * FROM $table WHERE serialid = ?" :
     ($table eq 'problem_reports') ? "SELECT * FROM $table WHERE reportid = ?" :
-    ($table eq 'additional_contents' || $table eq 'opac_news') ? "SELECT * FROM additional_contents WHERE idnew = ?" :
+    ($table eq 'additional_contents' || $table eq 'opac_news') ? "SELECT * FROM additional_contents_localizations WHERE id = ?" :
     ($table eq 'recalls') ? "SELECT * FROM $table WHERE recall_id = ?" :
     undef ;
     unless ($query) {
@@ -952,22 +982,22 @@ sub EnqueueLetter {
 =head2 SendQueuedMessages ([$hashref]) 
 
     my $sent = SendQueuedMessages({
-        letter_code => $letter_code,
+        message_id => $id,
         borrowernumber => $who_letter_is_for,
+        letter_code => $letter_code, # can be scalar or arrayref
+        type => $type, # can be scalar or arrayref
         limit => 50,
         verbose => 1,
-        type => 'sms',
+        where => $where,
     });
 
-Sends all of the 'pending' items in the message queue, unless
-parameters are passed.
+Sends 'pending' messages from the queue, based on parameters.
 
-The letter_code, borrowernumber and limit parameters are used
-to build a parameter set for _get_unsent_messages, thus limiting
-which pending messages will be processed. They are all optional.
+The (optional) message_id, borrowernumber, letter_code, type and where
+parameter are used to select which pending messages will be processed. The
+limit parameter determines the volume of results, i.e. sent messages.
 
-The verbose parameter can be used to generate debugging output.
-It is also optional.
+The optional verbose parameter can be used to generate debugging output.
 
 Returns number of messages sent.
 
@@ -975,33 +1005,45 @@ Returns number of messages sent.
 
 sub SendQueuedMessages {
     my $params = shift;
+    my $limit = $params->{limit};
+    my $where = $params->{where};
 
     Koha::Exceptions::BadParameter->throw("Parameter message_id cannot be empty if passed.")
         if ( exists( $params->{message_id} ) && !$params->{message_id} );
 
-    my $which_unsent_messages  = {
-        'message_id'     => $params->{'message_id'},
-        'limit'          => $params->{'limit'} // 0,
-        'borrowernumber' => $params->{'borrowernumber'} // q{},
-        'letter_code'    => $params->{'letter_code'} // q{},
-        'message_transport_type'           => $params->{'type'} // q{},
-    };
-    my $unsent_messages = _get_unsent_messages( $which_unsent_messages );
-    MESSAGE: foreach my $message ( @$unsent_messages ) {
-        my $message_object = Koha::Notice::Messages->find( $message->{message_id} );
+    my $smtp_transports = {};
+
+    my $count_messages = 0;
+    my $unsent_messages = Koha::Notice::Messages->search({
+        status  => 'pending',
+        $params->{message_id} ? ( message_id => $params->{message_id} ) : (),
+        $params->{borrowernumber} ? ( borrowernumber => $params->{borrowernumber} ) : (),
+        # Check for scalar or array in letter_code and type
+        ref($params->{letter_code}) && @{$params->{letter_code}} ? ( letter_code => $params->{letter_code} ) : (),
+        !ref($params->{letter_code}) && $params->{letter_code} ? ( letter_code => $params->{letter_code} ) : (),
+        ref($params->{type}) && @{$params->{type}} ? ( message_transport_type => $params->{type} ) : (), #TODO Existing inconsistency
+        !ref($params->{type}) && $params->{type} ? ( message_transport_type => $params->{type} ) : (), #TODO Existing inconsistency
+    });
+    $unsent_messages = $unsent_messages->search( \$where ) if $where;
+
+    $domain_limits = Koha::Notice::Util->load_domain_limits; # (re)initialize per run
+    while( ( my $message_object = $unsent_messages->next ) && ( !$limit || $count_messages < $limit ) ) {
+        my $message = $message_object->unblessed;
+
         # If this fails the database is unwritable and we won't manage to send a message that continues to be marked 'pending'
         $message_object->make_column_dirty('status');
         return unless $message_object->store;
 
         # warn Data::Dumper->Dump( [ $message ], [ 'message' ] );
-        warn sprintf( 'sending %s message to patron: %s',
+        warn sprintf( 'Processing %s message to patron: %s',
                       $message->{'message_transport_type'},
                       $message->{'borrowernumber'} || 'Admin' )
           if $params->{'verbose'};
         # This is just begging for subclassing
-        next MESSAGE if ( lc($message->{'message_transport_type'}) eq 'rss' );
+        next if ( lc($message->{'message_transport_type'}) eq 'rss' );
         if ( lc( $message->{'message_transport_type'} ) eq 'email' ) {
-            _send_message_by_email( $message, $params->{'username'}, $params->{'password'}, $params->{'method'} );
+            my $rv = _send_message_by_email( $message, $params->{'username'}, $params->{'password'}, $params->{'method'}, $smtp_transports );
+            $count_messages++ if $rv;
         }
         elsif ( lc( $message->{'message_transport_type'} ) eq 'sms' ) {
             if ( C4::Context->preference('SMSSendDriver') eq 'Email' ) {
@@ -1010,12 +1052,12 @@ sub SendQueuedMessages {
                 unless ( $sms_provider ) {
                     warn sprintf( "Patron %s has no sms provider id set!", $message->{'borrowernumber'} ) if $params->{'verbose'};
                     _set_message_status( { message_id => $message->{'message_id'}, status => 'failed' } );
-                    next MESSAGE;
+                    next;
                 }
                 unless ( $patron->smsalertnumber ) {
                     _set_message_status( { message_id => $message->{'message_id'}, status => 'failed' } );
                     warn sprintf( "No smsalertnumber found for patron %s!", $message->{'borrowernumber'} ) if $params->{'verbose'};
-                    next MESSAGE;
+                    next;
                 }
                 $message->{to_address}  = $patron->smsalertnumber; #Sometime this is set to email - sms should always use smsalertnumber
                 $message->{to_address} .= '@' . $sms_provider->domain();
@@ -1028,13 +1070,15 @@ sub SendQueuedMessages {
                 }
 
                 _update_message_to_address($message->{'message_id'}, $message->{to_address});
-                _send_message_by_email( $message, $params->{'username'}, $params->{'password'}, $params->{'method'} );
+                my $rv = _send_message_by_email( $message, $params->{'username'}, $params->{'password'}, $params->{'method'} );
+                $count_messages++ if $rv;
             } else {
-                _send_message_by_sms( $message );
+                my $rv = _send_message_by_sms( $message );
+                $count_messages++ if $rv;
             }
         }
     }
-    return scalar( @$unsent_messages );
+    return $count_messages;
 }
 
 =head2 GetRSSMessages
@@ -1090,7 +1134,7 @@ sub GetQueuedMessages {
 
     my $dbh = C4::Context->dbh();
     my $statement = << 'ENDSQL';
-SELECT message_id, borrowernumber, subject, content, message_transport_type, status, time_queued, updated_on, failure_code
+SELECT message_id, borrowernumber, subject, content, message_transport_type, status, time_queued, updated_on, failure_code, from_address, to_address, cc_address
 FROM message_queue
 ENDSQL
 
@@ -1256,7 +1300,7 @@ sub _get_unsent_messages {
 
     my $dbh = C4::Context->dbh();
     my $statement = qq{
-        SELECT mq.message_id, mq.borrowernumber, mq.subject, mq.content, mq.message_transport_type, mq.status, mq.time_queued, mq.from_address, mq.reply_address, mq.to_address, mq.content_type, b.branchcode, mq.letter_code, mq.failure_code
+        SELECT mq.message_id, mq.borrowernumber, mq.subject, mq.content, mq.message_transport_type, mq.status, mq.time_queued, mq.from_address, mq.reply_address, mq.to_address, mq.content_type, b.branchcode, mq.letter_code, mq.failure_code, mq.letter_id
         FROM message_queue mq
         LEFT JOIN borrowers b ON b.borrowernumber = mq.borrowernumber
         WHERE status = ?
@@ -1288,6 +1332,9 @@ sub _get_unsent_messages {
             $statement .= ' AND message_id = ?';
             push @query_params, $params->{message_id};
         }
+        if ( $params->{where} ) {
+            $statement .= " AND $params->{where} ";
+        }
         if ( $params->{'limit'} ) {
             $statement .= ' limit ? ';
             push @query_params, $params->{'limit'};
@@ -1301,12 +1348,25 @@ sub _get_unsent_messages {
 
 sub _send_message_by_email {
     my $message = shift or return;
-    my ($username, $password, $method) = @_;
+    my ( $username, $password, $method, $smtp_transports ) = @_;
 
     my $patron = Koha::Patrons->find( $message->{borrowernumber} );
     my $to_address = $message->{'to_address'};
+    my $cc_address;
+    my @guarantor_address;
+    my $count_guarantor_address;
+    if (C4::Context->preference('RedirectGuaranteeEmail') && $patron) {
+        # Get guarantor addresses
+        my $guarantor_relationships = $patron->guarantor_relationships;
+        my @guarantors              = $guarantor_relationships->guarantors->as_list;
+        foreach my $guarantor (@guarantors) {
+            my $address = $guarantor->notice_email_address;
+            push( @guarantor_address, $address ) if $address;
+        }
+        $count_guarantor_address = scalar @guarantor_address;
+    }
     unless ($to_address) {
-        unless ($patron) {
+        if (!$patron) {
             warn "FAIL: No 'to_address' and INVALID borrowernumber ($message->{borrowernumber})";
             _set_message_status(
                 {
@@ -1317,10 +1377,11 @@ sub _send_message_by_email {
             );
             return;
         }
-        $to_address = $patron->notice_email_address;
-        unless ($to_address) {  
-            # warn "FAIL: No 'to_address' and no email for " . ($member->{surname} ||'') . ", borrowernumber ($message->{borrowernumber})";
-            # warning too verbose for this more common case?
+        if ($patron) {
+            $to_address = $patron->notice_email_address;
+        }
+        if (!$to_address && !$count_guarantor_address) {
+            warn "FAIL: No 'to_address', email address or guarantors email address for borrowernumber ($message->{borrowernumber})";
             _set_message_status(
                 {
                     message_id   => $message->{'message_id'},
@@ -1330,6 +1391,19 @@ sub _send_message_by_email {
             );
             return;
         }
+        if ( !$to_address && $count_guarantor_address ) {
+            $to_address = shift @guarantor_address;
+        }
+    }
+
+    $cc_address = join( ',', @guarantor_address );
+    _update_message_cc_address( $message->{'message_id'}, $cc_address );
+    # Skip this message if we exceed domain limits in this run
+    if( Koha::Notice::Util->exceeds_limit({ to => $to_address, limits => $domain_limits }) ) {
+        # Save the to_address if you delay the message so that we dont need to look it up again
+        _update_message_to_address( $message->{'message_id'}, $to_address )
+            if !$message->{to_address};
+        return;
     }
 
     my $subject = $message->{'subject'};
@@ -1343,6 +1417,7 @@ sub _send_message_by_email {
     my $branch_returnpath = undef;
     my $library;
 
+    $patron //= Koha::Patrons->find( $message->{borrowernumber} ); # we might already found him
     if ($patron) {
         $library           = $patron->library;
         $branch_email      = $library->from_email_address;
@@ -1377,10 +1452,21 @@ sub _send_message_by_email {
                 ? ( bcc => C4::Context->preference('NoticeBcc') )
                 : ()
             ),
+            (
+                $cc_address
+                ? ( cc => $cc_address )
+                : ()
+            ),
             from     => $from_address,
             reply_to => $message->{'reply_address'} || $branch_replyto,
             sender   => $branch_returnpath,
-            subject  => "" . $message->{subject}
+            subject  => "" . $message->{subject},
+            (
+                $message->{letter_id}
+                ? ( template_id => $message->{letter_id} )
+                : ()
+            ),
+            message_id => $message->{id}
         };
 
         if ( $message->{'content_type'} && $message->{'content_type'} eq SERIALIZED_EMAIL_CONTENT_TYPE ) {
@@ -1443,8 +1529,15 @@ sub _send_message_by_email {
       if !$message->{to_address}
       || $message->{to_address} ne $email->email->header('To');
 
+    $smtp_transports->{ $smtp_server->id // 'default' } ||= $smtp_server->transport;
+    my $smtp_transport = $smtp_transports->{ $smtp_server->id // 'default' };
+
+    _update_message_from_address( $message->{'message_id'}, $email->email->header('From') )
+        if !$message->{from_address}
+        || $message->{from_address} ne $email->email->header('From');
+
     try {
-        $email->send_or_die({ transport => $smtp_server->transport });
+        $email->send_or_die({ transport => $smtp_transport });
 
         _set_message_status(
             {
@@ -1476,8 +1569,8 @@ sub _wrap_html {
     $css = qq{<link rel="stylesheet" type="text/css" href="$css">} if $css;
     return <<EOS;
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
-    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html lang="en" xml:lang="en" xmlns="http://www.w3.org/1999/xhtml">
+    "https://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html lang="en" xml:lang="en" xmlns="https://www.w3.org/1999/xhtml">
 <head>
 <title>$title</title>
 <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
@@ -1529,7 +1622,7 @@ sub _send_message_by_sms {
         return;
     }
 
-    my $success = C4::SMS->send_sms(
+    my ( $success, $error ) = C4::SMS->send_sms(
         {
             destination => $patron->smsalertnumber,
             message     => $message->{'content'},
@@ -1550,7 +1643,7 @@ sub _send_message_by_sms {
             {
                 message_id   => $message->{'message_id'},
                 status       => 'failed',
-                failure_code => 'NO_NOTES'
+                failure_code => $error || 'NO_NOTES',
             }
         );
     }
@@ -1570,6 +1663,12 @@ sub _update_message_from_address {
     $dbh->do('UPDATE message_queue SET from_address = ? WHERE message_id = ?', undef, ($from_address, $message_id));
 }
 
+sub _update_message_cc_address {
+    my ($message_id, $cc_address) = @_;
+    my $dbh = C4::Context->dbh();
+    $dbh->do('UPDATE message_queue SET cc_address = ? WHERE message_id = ?', undef, ($cc_address, $message_id));
+}
+
 sub _set_message_status {
     my $params = shift or return;
 
@@ -1592,9 +1691,9 @@ sub _process_tt {
     my $content    = $params->{content};
     my $tables     = $params->{tables};
     my $loops      = $params->{loops};
-    my $objects    = $params->{objects} || {};
+    my $objects    = $params->{objects}    || {};
     my $substitute = $params->{substitute} || {};
-    my $lang = defined($params->{lang}) && $params->{lang} ne 'default' ? $params->{lang} : 'en';
+    my $lang       = $params->{lang} // 'en';
     my ($theme, $availablethemes);
 
     my $htdocs = C4::Context->config('intrahtdocs');
@@ -1706,16 +1805,16 @@ sub _get_tt_params {
             pk       => 'itemnumber',
         },
         additional_contents => {
-            module   => 'Koha::AdditionalContents',
+            module   => 'Koha::AdditionalContentsLocalizations',
             singular => 'additional_content',
             plural   => 'additional_contents',
-            pk       => 'idnew',
+            pk       => 'id',
         },
         opac_news => {
-            module   => 'Koha::AdditionalContents',
+            module   => 'Koha::AdditionalContentsLocalizations',
             singular => 'news',
             plural   => 'news',
-            pk       => 'idnew',
+            pk       => 'id',
         },
         aqorders => {
             module   => 'Koha::Acquisition::Orders',
@@ -1747,6 +1846,18 @@ sub _get_tt_params {
             plural   => 'suggestions',
             pk       => 'suggestionid',
         },
+        tickets => {
+            module   => 'Koha::Tickets',
+            singular => 'ticket',
+            plural   => 'tickets',
+            pk       => 'id',
+        },
+        ticket_updates => {
+            module   => 'Koha::Ticket::Updates',
+            singular => 'ticket_update',
+            plural   => 'ticket_updates',
+            pk       => 'id',
+        },
         issues => {
             module   => 'Koha::Checkouts',
             singular => 'checkout',
@@ -1772,11 +1883,17 @@ sub _get_tt_params {
             fk       => 'verification_token',
         },
         illrequests => {
-            module   => 'Koha::Illrequests',
+            module   => 'Koha::ILL::Requests',
             singular => 'illrequest',
             plural   => 'illrequests',
             pk       => 'illrequest_id'
-        }
+        },
+        preservation_train_items => {
+            module   => 'Koha::Preservation::Train::Items',
+            singular => 'train_item',
+            plural   => 'train_items',
+            pk       => 'train_item_id'
+        },
     };
 
     my $dbh = C4::Context->dbh;
