@@ -288,10 +288,12 @@ sub CalcFine {
         && $amount > $issuing_rule->{overduefinescap};
 
     # This must be moved to Koha::Item (see also similar code in C4::Accounts::chargelostitem
-    $item->{replacementprice} ||= $itemtype->defaultreplacecost
-      if $itemtype
-      && ( ! defined $item->{replacementprice} || $item->{replacementprice} == 0 )
-      && C4::Context->preference("useDefaultReplacementCost");
+    if (   $itemtype
+        && ( !defined $item->{replacementprice} || $item->{replacementprice} + .0 == 0 )
+        && C4::Context->preference("useDefaultReplacementCost") )
+    {
+        $item->{replacementprice} = $itemtype->defaultreplacecost;
+    }
 
     $amount = $item->{replacementprice} if ( $issuing_rule->{cap_fine_to_replacement_price} && $item->{replacementprice} && $amount > $item->{replacementprice} );
 
@@ -683,7 +685,7 @@ sub GetBranchcodesWithOverdueRules {
         WHERE delay1 IS NOT NULL
         ORDER BY branchcode
     |);
-    if ( $branchcodes->[0] eq '' ) {
+    if ( defined $branchcodes->[0] && $branchcodes->[0] eq '' ) {
         # If a default rule exists, all branches should be returned
         return Koha::Libraries->search({}, { order_by => 'branchname' })->get_column('branchcode');
     }
