@@ -155,6 +155,9 @@ our $RULE_KINDS = {
     norenewalbefore => {
         scope => [ 'branchcode', 'categorycode', 'itemtype' ],
     },
+    noautorenewalbefore => {
+        scope => [ 'branchcode', 'categorycode', 'itemtype' ],
+    },
     onshelfholds => {
         scope => [ 'branchcode', 'categorycode', 'itemtype' ],
     },
@@ -210,6 +213,9 @@ our $RULE_KINDS = {
     recall_shelf_time => {
         scope => [ 'branchcode', 'categorycode', 'itemtype' ],
     },
+    holds_pickup_period => {
+        scope => [ 'branchcode', 'categorycode', 'itemtype' ],
+    },
     # Not included (deprecated?):
     #   * accountsent
     #   * reservecharge
