@@ -25,10 +25,12 @@ use Mojo::JSON;
 use Scalar::Util qw( blessed looks_like_number );
 use Try::Tiny qw( catch try );
 use List::MoreUtils qw( any );
+use DateTime::Format::MySQL;
 
 use Koha::Database;
+use Koha::DateTime::Format::RFC3339;
+use Koha::DateTime::Format::SQL;
 use Koha::Exceptions::Object;
-use Koha::DateUtils qw( dt_from_string output_pref );
 use Koha::Object::Message;
 
 =head1 NAME
@@ -231,11 +233,32 @@ Returns:
 sub delete {
     my ($self) = @_;
 
-    my $deleted = $self->_result()->delete;
+    my $deleted;
+
+    try {
+        $deleted = $self->_result()->delete;
+    } catch {
+        if ( ref($_) eq 'DBIx::Class::Exception' ) {
+            if ( $_->{msg} =~
+                /Cannot delete or update a parent row\: a foreign key constraint fails \(\`(?<database>.*?)\`\.\`(?<table>.*?)\`, CONSTRAINT \`(?<constraint>.*?)\` FOREIGN KEY \(\`(?<fk>.*?)\`\) REFERENCES \`.*\` \(\`(?<column>.*?)\`\)/
+                )
+            {
+                Koha::Exceptions::Object::FKConstraintDeletion->throw(
+                    column     => $+{column},
+                    constraint => $+{constraint},
+                    fk         => $+{fk},
+                    table      => $+{table},
+                );
+            }
+            $_->rethrow();
+        }
+    };
+
     if ( ref $deleted ) {
-        my $object_class  = Koha::Object::_get_object_class( $self->_result->result_class );
+        my $object_class = Koha::Object::_get_object_class( $self->_result->result_class );
         $deleted = $object_class->_new_from_dbic($deleted);
     }
+
     return $deleted;
 }
 
@@ -405,7 +428,6 @@ sub TO_JSON {
         elsif ( _numeric_column_type( $columns_info->{$col}->{data_type} )
             and looks_like_number( $unblessed->{$col} )
         ) {
-
             # TODO: Remove once the solution for
             # https://github.com/perl5-dbi/DBD-mysql/issues/212
             # is ported to whatever distro we support by that time
@@ -415,7 +437,6 @@ sub TO_JSON {
         elsif ( _decimal_column_type( $columns_info->{$col}->{data_type} )
             and looks_like_number( $unblessed->{$col} )
         ) {
-
             # TODO: Remove once the solution for
             # https://github.com/perl5-dbi/DBD-mysql/issues/212
             # is ported to whatever distro we support by that time
@@ -425,10 +446,8 @@ sub TO_JSON {
         elsif ( _datetime_column_type( $columns_info->{$col}->{data_type} ) ) {
             eval {
                 return unless $unblessed->{$col};
-                $unblessed->{$col} = output_pref({
-                    dateformat => 'rfc3339',
-                    dt         => dt_from_string($unblessed->{$col}, 'sql'),
-                });
+                my $dt = Koha::DateTime::Format::SQL->parse_datetime( $unblessed->{$col} );
+                $unblessed->{$col} = Koha::DateTime::Format::RFC3339->format_datetime($dt);
             };
         }
     }
@@ -551,6 +570,7 @@ Returns a representation of the object, suitable for API output.
 
 sub to_api {
     my ( $self, $params ) = @_;
+
     my $json_object = $self->TO_JSON;
 
     # Make sure we duplicate the $params variable to avoid
@@ -582,6 +602,23 @@ sub to_api {
         }
     }
 
+    # Remove forbidden attributes if required (including their coded values)
+    if ( !$self->is_accessible($params) ) {
+        for my $field ( keys %{$json_object} ) {
+            unless ( any { $_ eq $field } @{ $self->unredact_list } ) {
+                $json_object->{$field} = undef;
+            }
+        }
+
+        if ($strings) {
+            foreach my $field ( keys %{$string_map} ) {
+                unless ( any { $_ eq $field } @{ $self->unredact_list } ) {
+                    $string_map->{$field} = undef;
+                }
+            }
+        }
+    }
+
     my $to_api_mapping = $self->to_api_mapping;
 
     # Rename attributes and coded values if there's a mapping
@@ -714,6 +751,23 @@ sub public_read_list
     return [];
 }
 
+=head3 unredact_list
+
+    my @unredact_list = @{$object->unredact_list};
+
+Generic method that returns the list of database columns that are allowed to
+be passed to render objects on the API when the user making the request should
+not ordinarily have unrestricted access to the data (as returned by the is_accesible method).
+
+Note: this only returns an empty I<arrayref>. Each class should have its
+own implementation.
+
+=cut
+
+sub unredact_list {
+    return [];
+}
+
 =head3 from_api_mapping
 
     my $mapping = $object->from_api_mapping;
@@ -799,19 +853,21 @@ sub attributes_from_api {
             $value = ( $value ) ? 1 : 0;
         }
         elsif ( _date_or_datetime_column_type( $columns_info->{$koha_field_name}->{data_type} ) ) {
-            try {
-                if ( $columns_info->{$koha_field_name}->{data_type} eq 'date' ) {
-                    $value = $dtf->format_date(dt_from_string($value, 'iso'))
-                        if defined $value;
-                }
-                else {
-                    $value = $dtf->format_datetime(dt_from_string($value, 'rfc3339'))
-                        if defined $value;
+            if (defined $value) {
+                try {
+                    if ( $columns_info->{$koha_field_name}->{data_type} eq 'date' ) {
+                        my $dt = DateTime::Format::MySQL->parse_date($value);
+                        $value = $dtf->format_date($dt);
+                    }
+                    else {
+                        my $dt = Koha::DateTime::Format::RFC3339->parse_datetime($value);
+                        $value = $dtf->format_datetime($dt);
+                    }
                 }
+                catch {
+                    Koha::Exceptions::BadParameter->throw( parameter => $key );
+                };
             }
-            catch {
-                Koha::Exceptions::BadParameter->throw( parameter => $key );
-            };
         }
 
         $params->{$koha_field_name} = $value;
@@ -906,15 +962,26 @@ sub AUTOLOAD {
     $method =~ s/.*://;
 
     my @columns = @{$self->_columns()};
-    # Using direct setter/getter like $item->barcode() or $item->barcode($barcode);
     if ( grep { $_ eq $method } @columns ) {
-        if ( @_ ) {
-            $self->_result()->set_column( $method, @_ );
-            return $self;
-        } else {
-            my $value = $self->_result()->get_column( $method );
-            return $value;
+
+        # Lazy definition of get/set accessors like $item->barcode; note that it contains $method
+        my $accessor = sub {
+            my $self = shift;
+            if (@_) {
+                $self->_result()->set_column( $method, @_ );
+                return $self;
+            } else {
+                return $self->_result()->get_column($method);
+            }
+        };
+        # If called from child class as $self->SUPER-><accessor_name>
+        # $AUTOLOAD will contain ::SUPER which breaks method lookup
+        # therefore we cannot write those entries into the symbol table
+        unless ( $AUTOLOAD =~ /::SUPER::/ ) {
+            no strict 'refs'; ## no critic (strict)
+            *{$AUTOLOAD} = $accessor;
         }
+        return $accessor->( $self, @_ );
     }
 
     my @known_methods = qw( is_changed id in_storage get_column discard_changes make_column_dirty );
@@ -924,7 +991,6 @@ sub AUTOLOAD {
         show_trace => 1
     ) unless grep { $_ eq $method } @known_methods;
 
-
     my $r = eval { $self->_result->$method(@_) };
     if ( $@ ) {
         Koha::Exceptions::Object->throw( ref($self) . "::$method generated this error: " . $@ );
@@ -972,6 +1038,20 @@ sub _handle_to_api_child {
     return $res;
 }
 
+=head3 is_accessible
+
+    if ( $object->is_accessible ) { ... }
+
+Stub method that is expected to be overloaded (if required) by implementing classes.
+
+=cut
+
+sub is_accessible {
+    my ($self) = @_;
+
+    return 1;
+}
+
 sub DESTROY { }
 
 =head1 AUTHOR
