@@ -29,7 +29,7 @@ use Koha::Patron;
 use Koha::Exceptions::Patron;
 use Koha::Patron::Categories;
 
-use base qw(Koha::Objects);
+use base qw(Koha::Objects::Mixin::ExtendedAttributes Koha::Objects);
 
 =head1 NAME
 
@@ -205,8 +205,7 @@ sub filter_by_expiration_date {
     return $class->filter_by_last_update(
         {
             timestamp_column_name => 'dateexpiry',
-            days                  => $params->{days} || 0,
-            days_inclusive        => 1,
+            min_days              => $params->{days} || 0,
         }
     );
 }
@@ -400,6 +399,7 @@ sub update_category_to {
     my ( $self, $params ) = @_;
     my $counter = 0;
     while( my $patron = $self->next ) {
+        $patron->discard_changes;
         $counter++;
         $patron->categorycode($params->{category})->store();
     }
@@ -566,6 +566,19 @@ sub filter_by_have_permission {
     );
 }
 
+=head3 extended_attributes_config
+
+=cut
+
+sub extended_attributes_config {
+    my ($self) = @_;
+    return {
+        'id_field'     => { 'foreign' => 'borrowernumber', 'self' => 'borrowernumber' },
+        'key_field'    => 'code',
+        'schema_class' => 'Koha::Schema::Result::BorrowerAttribute',
+    };
+}
+
 =head3 _type
 
 =cut
