@@ -20,11 +20,13 @@ use Modern::Perl;
 use Mojo::Base 'Mojolicious::Controller';
 
 use Koha::Biblios;
+use Koha::DateUtils;
 use Koha::Ratings;
 use Koha::RecordProcessor;
 use C4::Biblio qw( DelBiblio AddBiblio ModBiblio );
 use C4::Search qw( FindDuplicate );
 
+use C4::Auth qw( haspermission );
 use C4::Barcodes::ValueBuilder;
 use C4::Context;
 
@@ -34,6 +36,7 @@ use List::MoreUtils qw( any );
 use MARC::Record::MiJ;
 
 use Try::Tiny qw( catch try );
+use JSON qw( decode_json );
 
 =head1 API
 
@@ -52,23 +55,17 @@ sub get {
     $attributes = { prefetch => [ 'metadata' ] } # don't prefetch metadata if not needed
         unless $c->req->headers->accept =~ m/application\/json/;
 
-    my $biblio = Koha::Biblios->find( { biblionumber => $c->validation->param('biblio_id') }, $attributes );
+    my $biblio = Koha::Biblios->find( { biblionumber => $c->param('biblio_id') }, $attributes );
 
-    unless ( $biblio ) {
-        return $c->render(
-            status  => 404,
-            openapi => {
-                error => "Object not found."
-            }
-        );
-    }
+    return $c->render_resource_not_found("Bibliographic record")
+        unless $biblio;
 
     return try {
 
         if ( $c->req->headers->accept =~ m/application\/json/ ) {
             return $c->render(
                 status => 200,
-                json   => $biblio->to_api
+                json   => $c->objects->to_api($biblio),
             );
         }
         else {
@@ -124,14 +121,10 @@ Controller function that handles deleting a biblio object
 sub delete {
     my $c = shift->openapi->valid_input or return;
 
-    my $biblio = Koha::Biblios->find( $c->validation->param('biblio_id') );
+    my $biblio = Koha::Biblios->find( $c->param('biblio_id') );
 
-    if ( not defined $biblio ) {
-        return $c->render(
-            status  => 404,
-            openapi => { error => "Object not found" }
-        );
-    }
+    return $c->render_resource_not_found("Bibliographic record")
+        unless $biblio;
 
     return try {
         my $error = DelBiblio( $biblio->id );
@@ -143,7 +136,7 @@ sub delete {
             );
         }
         else {
-            return $c->render( status => 204, openapi => "" );
+            return $c->render_resource_deleted;
         }
     }
     catch {
@@ -161,17 +154,11 @@ sub get_public {
     my $c = shift->openapi->valid_input or return;
 
     my $biblio = Koha::Biblios->find(
-        { biblionumber => $c->validation->param('biblio_id') },
+        { biblionumber => $c->param('biblio_id') },
         { prefetch     => ['metadata'] } );
 
-    unless ($biblio) {
-        return $c->render(
-            status  => 404,
-            openapi => {
-                error => "Object not found."
-            }
-        );
-    }
+    return $c->render_resource_not_found("Bibliographic record")
+        unless $biblio;
 
     return try {
 
@@ -185,14 +172,8 @@ sub get_public {
         # unless there's a logged in user, and there's an exception for it's
         # category
         unless ( $patron and $patron->category->override_hidden_items ) {
-            if ( $biblio->hidden_in_opac({ rules => $opachiddenitems_rules }) )
-            {
-                return $c->render(
-                    status  => 404,
-                    openapi => {
-                        error => "Object not found."
-                    }
-                );
+            if ( $biblio->hidden_in_opac( { rules => $opachiddenitems_rules } ) ) {
+                return $c->render_resource_not_found("Bibliographic record");
             }
         }
 
@@ -245,6 +226,34 @@ sub get_public {
     };
 }
 
+
+=head3 get_bookings
+
+Controller function that handles retrieving biblio's bookings
+
+=cut
+
+sub get_bookings {
+    my $c = shift->openapi->valid_input or return;
+
+    my $biblio = Koha::Biblios->find( { biblionumber => $c->param('biblio_id') }, { prefetch => ['bookings'] } );
+
+    return $c->render_resource_not_found("Bibliographic record")
+        unless $biblio;
+
+    return try {
+
+        my $bookings_rs = $biblio->bookings;
+        my $bookings    = $c->objects->search($bookings_rs);
+        return $c->render(
+            status  => 200,
+            openapi => $bookings
+        );
+    } catch {
+        $c->unhandled_exception($_);
+    };
+}
+
 =head3 get_items
 
 Controller function that handles retrieving biblio's items
@@ -254,31 +263,189 @@ Controller function that handles retrieving biblio's items
 sub get_items {
     my $c = shift->openapi->valid_input or return;
 
-    my $biblio = Koha::Biblios->find( { biblionumber => $c->validation->param('biblio_id') }, { prefetch => ['items'] } );
+    my $biblio        = Koha::Biblios->find( { biblionumber => $c->param('biblio_id') }, { prefetch => ['items'] } );
+    my $bookable_only = $c->param('bookable');
 
-    unless ( $biblio ) {
-        return $c->render(
-            status  => 404,
-            openapi => {
-                error => "Object not found."
-            }
-        );
-    }
+    return $c->render_resource_not_found("Bibliographic record")
+        unless $biblio;
 
     return try {
 
-        my $items_rs = $biblio->items;
-        my $items    = $c->objects->search( $items_rs );
+        # FIXME We need to order_by serial.publisheddate if we have _order_by=+me.serial_issue_number
+        # FIXME Do we always need host_items => 1 or depending on a flag?
+        # FIXME Should we prefetch => ['issue','branchtransfer']?
+        my $items_rs = $biblio->items( { host_items => 1 } )->search_ordered( {}, { join => 'biblioitem' } );
+        $items_rs = $items_rs->filter_by_bookable if $bookable_only;
+        # FIXME We need to order_by serial.publisheddate if we have _order_by=+me.serial_issue_number
+        my $items = $c->objects->search($items_rs);
+
         return $c->render(
             status  => 200,
             openapi => $items
         );
-    }
-    catch {
+    } catch {
         $c->unhandled_exception($_);
     };
 }
 
+=head3 add_item
+
+Controller function that handles creating a biblio's item
+
+=cut
+
+sub add_item {
+    my $c = shift->openapi->valid_input or return;
+
+    try {
+        my $biblio_id = $c->param('biblio_id');
+        my $biblio    = Koha::Biblios->find( $biblio_id );
+
+        return $c->render_resource_not_found("Bibliographic record")
+            unless $biblio;
+
+        my $body = $c->req->json;
+
+        $body->{biblio_id} = $biblio_id;
+
+        # Don't save extended subfields yet. To be done in another bug.
+        $body->{extended_subfields} = undef;
+
+        my $item = Koha::Item->new_from_api($body);
+
+        if ( !defined $item->barcode ) {
+
+            # FIXME This should be moved to Koha::Item->store
+            my $autoBarcode = C4::Context->preference('autoBarcode');
+            my $barcode     = '';
+
+            if ( !$autoBarcode || $autoBarcode eq 'OFF' ) {
+                #We do nothing
+            }
+            elsif ( $autoBarcode eq 'incremental' ) {
+                ($barcode) =
+                  C4::Barcodes::ValueBuilder::incremental::get_barcode;
+            }
+            elsif ( $autoBarcode eq 'annual' ) {
+                my $year = Koha::DateUtils::dt_from_string()->year();
+                ($barcode) =
+                  C4::Barcodes::ValueBuilder::annual::get_barcode(
+                    { year => $year } );
+            }
+            elsif ( $autoBarcode eq 'hbyymmincr' ) {
+
+                # Generates a barcode where
+                #  hb = home branch Code,
+                #  yymm = year/month catalogued,
+                #  incr = incremental number,
+                #  reset yearly -fbcit
+                my $now        = Koha::DateUtils::dt_from_string();
+                my $year       = $now->year();
+                my $month      = $now->month();
+                my $homebranch = $item->homebranch // '';
+                ($barcode) =
+                  C4::Barcodes::ValueBuilder::hbyymmincr::get_barcode(
+                    { year => $year, mon => $month } );
+                $barcode = $homebranch . $barcode;
+            }
+            elsif ( $autoBarcode eq 'EAN13' ) {
+
+                # not the best, two catalogers could add the same
+                # barcode easily this way :/
+                my $query = "select max(abs(barcode)) from items";
+                my $dbh   = C4::Context->dbh;
+                my $sth   = $dbh->prepare($query);
+                $sth->execute();
+                my $nextnum;
+                while ( my ($last) = $sth->fetchrow_array ) {
+                    $nextnum = $last;
+                }
+                my $ean = CheckDigits('ean');
+                if ( $ean->is_valid($nextnum) ) {
+                    my $next = $ean->basenumber($nextnum) + 1;
+                    $nextnum = $ean->complete($next);
+                    $nextnum =
+                      '0' x ( 13 - length($nextnum) ) . $nextnum;    # pad zeros
+                }
+                else {
+                    warn "ERROR: invalid EAN-13 $nextnum, using increment";
+                    $nextnum++;
+                }
+                $barcode = $nextnum;
+            }
+            else {
+                warn "ERROR: unknown autoBarcode: $autoBarcode";
+            }
+            $item->barcode($barcode) if $barcode;
+        }
+
+        $item->store->discard_changes;
+
+        $c->render(
+            status  => 201,
+            openapi => $c->objects->to_api($item),
+        );
+    }
+    catch {
+        if ( blessed $_ and $_->isa('Koha::Exceptions::Object::DuplicateID') ) {
+            return $c->render(
+                status  => 409,
+                openapi => { error => 'Duplicate barcode.' }
+            );
+        }
+        $c->unhandled_exception($_);
+    }
+}
+
+=head3 update_item
+
+Controller function that handles updating a biblio's item
+
+=cut
+
+sub update_item {
+    my $c = shift->openapi->valid_input or return;
+
+    try {
+        my $biblio_id = $c->param('biblio_id');
+        my $item_id   = $c->param('item_id');
+        my $biblio    = Koha::Biblios->find( { biblionumber => $biblio_id } );
+
+        return $c->render_resource_not_found("Bibliographic record")
+            unless $biblio;
+
+        my $item = $biblio->items->find({ itemnumber => $item_id });
+
+        return $c->render_resource_not_found("Item")
+            unless $item;
+
+        my $body = $c->req->json;
+
+        $body->{biblio_id} = $biblio_id;
+
+        # Don't save extended subfields yet. To be done in another bug.
+        $body->{extended_subfields} = undef;
+
+        $item->set_from_api($body);
+
+        $item->store->discard_changes;
+
+        $c->render(
+            status  => 200,
+            openapi => $c->objects->to_api($item),
+        );
+    }
+    catch {
+        if ( blessed $_ and $_->isa('Koha::Exceptions::Object::DuplicateID') ) {
+            return $c->render(
+                status  => 409,
+                openapi => { error => 'Duplicate barcode.' }
+            );
+        }
+        $c->unhandled_exception($_);
+    }
+}
+
 =head3 get_checkouts
 
 List Koha::Checkout objects
@@ -288,17 +455,14 @@ List Koha::Checkout objects
 sub get_checkouts {
     my $c = shift->openapi->valid_input or return;
 
-    my $checked_in = delete $c->validation->output->{checked_in};
+    my $checked_in = $c->param('checked_in');
+    $c->req->params->remove('checked_in');
 
     try {
-        my $biblio = Koha::Biblios->find( $c->validation->param('biblio_id') );
+        my $biblio = Koha::Biblios->find( $c->param('biblio_id') );
 
-        unless ($biblio) {
-            return $c->render(
-                status  => 404,
-                openapi => { error => 'Object not found' }
-            );
-        }
+        return $c->render_resource_not_found("Bibliographic record")
+            unless $biblio;
 
         my $checkouts =
           ($checked_in)
@@ -325,18 +489,13 @@ used for building the dropdown selector
 sub pickup_locations {
     my $c = shift->openapi->valid_input or return;
 
-    my $biblio_id = $c->validation->param('biblio_id');
-    my $biblio = Koha::Biblios->find( $biblio_id );
+    my $biblio = Koha::Biblios->find( $c->param('biblio_id') );
 
-    unless ($biblio) {
-        return $c->render(
-            status  => 404,
-            openapi => { error => "Biblio not found" }
-        );
-    }
+    return $c->render_resource_not_found("Bibliographic record")
+        unless $biblio;
 
-    my $patron_id = delete $c->validation->output->{patron_id};
-    my $patron    = Koha::Patrons->find( $patron_id );
+    my $patron = Koha::Patrons->find( $c->param('patron_id') );
+    $c->req->params->remove('patron_id');
 
     unless ($patron) {
         return $c->render(
@@ -371,6 +530,16 @@ sub pickup_locations {
             my $pickup_locations = $c->objects->search($pl_set);
             @response = map { $_->{needs_override} = Mojo::JSON->false; $_; } @{$pickup_locations};
         }
+        @response = map {
+            if ( exists $pl_set->{_pickup_location_items}->{ $_->{library_id} }
+                && ref $pl_set->{_pickup_location_items}->{ $_->{library_id} } eq 'ARRAY' )
+            {
+                $_->{pickup_items} = $pl_set->{_pickup_location_items}->{ $_->{library_id} };
+            } else {
+                $_->{pickup_items} = [];
+            }
+            $_;
+        } @response;
 
         return $c->render(
             status  => 200,
@@ -392,16 +561,13 @@ access.
 sub get_items_public {
     my $c = shift->openapi->valid_input or return;
 
-    my $biblio = Koha::Biblios->find( { biblionumber => $c->validation->param('biblio_id') }, { prefetch => ['items'] } );
+    my $biblio = Koha::Biblios->find(
+        $c->param('biblio_id'),
+        { prefetch => ['items'] }
+    );
 
-    unless ( $biblio ) {
-        return $c->render(
-            status  => 404,
-            openapi => {
-                error => "Object not found."
-            }
-        );
-    }
+    return $c->render_resource_not_found("Bibliographic record")
+        unless $biblio;
 
     return try {
 
@@ -429,16 +595,10 @@ Set rating for the logged in user
 sub set_rating {
     my $c = shift->openapi->valid_input or return;
 
-    my $biblio = Koha::Biblios->find( $c->validation->param('biblio_id') );
+    my $biblio = Koha::Biblios->find( $c->param('biblio_id') );
 
-    unless ($biblio) {
-        return $c->render(
-            status  => 404,
-            openapi => {
-                error => "Object not found."
-            }
-        );
-    }
+    $c->render_resource_not_found("Bibliographic record")
+        unless $biblio;
 
     my $patron = $c->stash('koha.user');
     unless ($patron) {
@@ -449,7 +609,7 @@ sub set_rating {
         );
     }
 
-    my $body   = $c->validation->param('body');
+    my $body         = $c->req->json;
     my $rating_value = $body->{rating};
 
     return try {
@@ -504,6 +664,19 @@ sub add {
         my $flavour = $headers->header('x-record-schema');
         $flavour //= C4::Context->preference('marcflavour');
 
+        my $record_source_id = $headers->header('x-record-source-id');
+
+        if ($record_source_id) {
+
+            # We've been passed a record source. Verify they are allowed to
+            unless ( haspermission( $c->stash('koha.user')->userid, { editcatalogue => 'set_record_sources' } ) ) {
+                return $c->render(
+                    status  => 403,
+                    openapi => { error => 'You do not have permission to set the record source' }
+                );
+            }
+        }
+
         my $record;
 
         my $frameworkcode = $headers->header('x-framework-id');
@@ -541,12 +714,11 @@ sub add {
             }
         ) unless !$duplicatebiblionumber || $confirm_not_duplicate;
 
-        my ( $biblionumber, $oldbibitemnum );
-            ( $biblionumber, $oldbibitemnum ) = AddBiblio( $record, $frameworkcode );
+        my ( $biblio_id ) = AddBiblio( $record, $frameworkcode, { record_source_id => $record_source_id } );
 
         $c->render(
             status  => 200,
-            openapi => { id => $biblionumber }
+            openapi => { id => $biblio_id }
         );
     }
     catch {
@@ -563,15 +735,10 @@ Controller function that handles modifying an biblio object
 sub update {
     my $c = shift->openapi->valid_input or return;
 
-    my $biblio_id = $c->param('biblio_id');
-    my $biblio    = Koha::Biblios->find($biblio_id);
+    my $biblio = Koha::Biblios->find( $c->param('biblio_id') );
 
-    if ( ! defined $biblio ) {
-        return $c->render(
-            status  => 404,
-            openapi => { error => "Object not found" }
-        );
-    }
+    $c->render_resource_not_found("Bibliographic record")
+        unless $biblio;
 
     try {
         my $headers = $c->req->headers;
@@ -606,11 +773,11 @@ sub update {
             );
         }
 
-        ModBiblio( $record, $biblio_id, $frameworkcode );
+        ModBiblio( $record, $biblio->id, $frameworkcode );
 
         $c->render(
             status  => 200,
-            openapi => { id => $biblio_id }
+            openapi => { id => $biblio->id }
         );
     }
     catch {
@@ -689,4 +856,77 @@ sub list {
     };
 }
 
+=head3 merge
+
+Controller function that handles merging two biblios. If an optional
+MARCXML is provided as the request body, this MARCXML replaces the
+bibliodata of the merge target biblio. Syntax format inside the request body
+must match with the Marc format used into Koha installation (MARC21 or UNIMARC)
+
+=cut
+
+sub merge {
+    my $c                = shift->openapi->valid_input or return;
+    my $ref_biblionumber = $c->param('biblio_id');
+    my $json             = decode_json( $c->req->body );
+    my $bn_merge         = $json->{'biblio_id_to_merge'};
+    my $framework        = $json->{'framework_to_use'} // q{};
+    my $rules            = $json->{'rules'} || q{override};
+    my $override_rec     = $json->{'datarecord'} // q{};
+
+    my $biblio = Koha::Biblios->find($ref_biblionumber);
+    if ( not defined $biblio ) {
+        return $c->render(
+            status => 404,
+            json   => { error => sprintf( "[%s] biblio to merge into not found", $ref_biblionumber ) }
+        );
+    }
+    my $frombib = Koha::Biblios->find($bn_merge);
+    if ( not defined $frombib ) {
+        return $c->render(
+            status => 404,
+            json   => { error => sprintf( "[%s] from which to merge not found", $bn_merge ) }
+        );
+    }
+
+    if ( ( $rules eq 'override_ext' ) && ( $override_rec eq '' ) ) {
+        return $c->render(
+            status => 404,
+            json   => {
+                error =>
+                    "With the rule 'override_ext' you need to insert a bib record in marc-in-json format into 'record' field."
+            }
+        );
+    }
+
+    if ( ( $rules eq 'override' ) && ( $framework ne '' ) ) {
+        return $c->render(
+            status => 404,
+            json   => { error => "With the rule 'override' you can not use the field 'framework_to_use'." }
+        );
+    }
+
+    return try {
+        if ( $rules eq 'override_ext' ) {
+            my $record = MARC::Record::MiJ->new_from_mij_structure($override_rec);
+            $record->encoding('UTF-8');
+            $framework ||= $biblio->frameworkcode;
+            my $chk = ModBiblio( $record, $ref_biblionumber, $framework );
+            if ( $chk != 1 ) { die "Error on ModBiblio"; }    # ModBiblio returns 1 if everything as gone well
+        }
+
+        $biblio->merge_with( [$bn_merge] );
+
+        $c->respond_to(
+            mij => {
+                status => 200,
+                format => 'mij',
+                data   => $biblio->metadata->record->to_mij
+            }
+        );
+    } catch {
+        $c->render( status => 400, json => { error => $@ } );
+    };
+}
+
 1;
