@@ -71,10 +71,10 @@ if ( C4::Context->preference('AcqEnableFiles') ) {
         tabletag => 'aqinvoices', recordid => $invoiceid );
 }
 
-if ( $op && $op eq 'close' ) {
+if ( $op && $op eq 'cud-close' ) {
     output_and_exit( $input, $cookie, $template, 'insufficient_permission' )
         unless $logged_in_patron->has_permission( { acquisition => 'edit_invoices' } );
-    my @invoiceid = $input->multi_param('invoiceid');
+    my @invoiceid = split( ',', scalar $input->param('invoiceid') );
     foreach my $invoiceid ( @invoiceid ) {
         CloseInvoice($invoiceid);
     }
@@ -84,10 +84,10 @@ if ( $op && $op eq 'close' ) {
         exit 0;
     }
 }
-elsif ( $op && $op eq 'reopen' ) {
+elsif ( $op && $op eq 'cud-reopen' ) {
     output_and_exit( $input, $cookie, $template, 'insufficient_permission' )
         unless $logged_in_patron->has_permission( { acquisition => 'reopen_closed_invoices' } );
-    my @invoiceid = $input->multi_param('invoiceid');
+    my @invoiceid = split( ',', scalar $input->param('invoiceid') );
     foreach my $invoiceid ( @invoiceid ) {
         ReopenInvoice($invoiceid);
     }
@@ -97,7 +97,7 @@ elsif ( $op && $op eq 'reopen' ) {
         exit 0;
     }
 }
-elsif ( $op && $op eq 'mod' ) {
+elsif ( $op && $op eq 'cud-mod' ) {
     my $shipmentcost       = $input->param('shipmentcost');
     my $shipment_budget_id = $input->param('shipment_budget_id');
     my $invoicenumber      = $input->param('invoicenumber');
@@ -141,7 +141,7 @@ elsif ( $op && $op eq 'mod' ) {
 
     $template->param( modified => 1 );
 }
-elsif ( $op && $op eq 'delete' ) {
+elsif ( $op && $op eq 'cud-delete' ) {
 
     output_and_exit( $input, $cookie, $template, 'insufficient_permission' )
         unless $logged_in_patron->has_permission( { acquisition => 'delete_invoices' } );
@@ -154,7 +154,7 @@ elsif ( $op && $op eq 'delete' ) {
         exit 0;
     }
 }
-elsif ( $op && $op eq 'del_adj' ) {
+elsif ( $op && $op eq 'cud-del_adj' ) {
 
     output_and_exit( $input, $cookie, $template, 'insufficient_permission' )
         unless $logged_in_patron->has_permission( { acquisition => 'edit_invoices' } );
@@ -180,7 +180,7 @@ elsif ( $op && $op eq 'del_adj' ) {
         $del_adj->delete();
     }
 }
-elsif ( $op && $op eq 'mod_adj' ) {
+elsif ( $op && $op eq 'cud-mod_adj' ) {
 
     output_and_exit( $input, $cookie, $template, 'insufficient_permission' )
         unless $logged_in_patron->has_permission( { acquisition => 'edit_invoices' } );
@@ -251,6 +251,7 @@ elsif ( $op && $op eq 'mod_adj' ) {
     }
 }
 
+my $active_currency = Koha::Acquisition::Currencies->get_active,
 my $details = GetInvoiceDetails($invoiceid);
 my $bookseller = Koha::Acquisition::Booksellers->find( $details->{booksellerid} );
 my @orders_loop = ();
@@ -262,6 +263,7 @@ my $total_quantity = 0;
 my $total_tax_excluded = 0;
 my $total_tax_included = 0;
 my $total_tax_value = 0;
+my $has_invoice_unitprice;
 foreach my $order (@$orders) {
     my $line = get_infos( $order, $bookseller);
 
@@ -282,6 +284,7 @@ foreach my $order (@$orders) {
     $total_tax_included += get_rounded_price($$line{total_tax_included});
 
     $line->{orderline} = $line->{parent_ordernumber};
+    $has_invoice_unitprice = 1 if $line->{invoice_currency} ne $active_currency->currency;
     push @orders_loop, $line;
 }
 
@@ -341,9 +344,10 @@ $template->param(
     total_tax_excluded_shipment => $total_tax_excluded + $shipmentcost,
     total_tax_included_shipment => $total_tax_included + $shipmentcost,
     invoiceincgst               => $bookseller->invoiceincgst,
-    currency                    => Koha::Acquisition::Currencies->get_active,
+    currency                    => $active_currency,
     budgets                     => $budget_loop,
     budget                      => GetBudget( $shipmentcost_budgetid ),
+    has_invoice_unitprice       => $has_invoice_unitprice,
 );
 
 defined( $invoice_files ) && $template->param( files => $invoice_files->GetFilesInfo() );
