@@ -24,6 +24,7 @@ use CGI;
 use C4::Auth qw( get_template_and_user );
 use C4::Output qw( output_html_with_http_headers );
 use Koha::Clubs;
+use Koha::Club::Enrollment::Fields;
 
 my $cgi = CGI->new;
 
@@ -38,12 +39,16 @@ my ( $template, $loggedinuser, $cookie ) = get_template_and_user(
 
 my $id             = $cgi->param('id');
 my $borrowernumber = $cgi->param('borrowernumber');
+my $enrollent_id   = scalar $cgi->param('enrollent_id');
 
 my $club = Koha::Clubs->find($id);
+my @club_enrollment_fields = Koha::Club::Enrollment::Fields->search({'club_enrollment_id'=> $enrollent_id})->as_list;
 
 $template->param(
-    club           => $club,
-    borrowernumber => $borrowernumber,
+    club                   => $club,
+    borrowernumber         => $borrowernumber,
+    enrollent_id           => $enrollent_id || 0,
+    club_enrollment_fields => \@club_enrollment_fields,
 );
 
 output_html_with_http_headers( $cgi, $cookie, $template->output );
