@@ -126,15 +126,15 @@ $(document).ready(function() {
         var holds = new Array();
         if ( ! holdsTable ) {
             var title;
-            holdsTable = $("#holds-table").dataTable($.extend(true, {}, dataTablesDefaults, {
-                "bAutoWidth": false,
-                "sDom": "rt",
+            holdsTable = KohaTable("holds-table", {
+                "autoWidth": false,
+                "dom": '<"table_controls"B>rt',
                 "columns": [
                     {
                         "data": { _: "reservedate_formatted", "sort": "reservedate" }
                     },
                     {
-                        "mDataProp": function ( oObj ) {
+                        "data": function ( oObj ) {
                             title = "<a href='/cgi-bin/koha/reserve/request.pl?biblionumber="
                                   + oObj.biblionumber
                                   + "'>"
@@ -168,12 +168,12 @@ $(document).ready(function() {
                         }
                     },
                     {
-                        "mDataProp": function( oObj ) {
+                        "data": function( oObj ) {
                             return oObj.itemcallnumber && oObj.itemcallnumber.escapeHtml() || "";
                         }
                     },
                     {
-                        "mDataProp": function( oObj ) {
+                        "data": function( oObj ) {
                             var data = "";
                             if ( oObj.itemtype ) {
                                 data += oObj.itemtype_description;
@@ -182,7 +182,7 @@ $(document).ready(function() {
                         }
                     },
                     {
-                        "mDataProp": function( oObj ) {
+                        "data": function( oObj ) {
                             var data = "";
                             if ( oObj.barcode ) {
                                 data += " <a href='/cgi-bin/koha/catalogue/moredetail.pl?biblionumber="
@@ -199,7 +199,7 @@ $(document).ready(function() {
                         }
                     },
                     {
-                        "mDataProp": function( oObj ) {
+                        "data": function( oObj ) {
                             if( oObj.branches.length > 1 && oObj.found !== 'W' && oObj.found !== 'T' ){
                                 var branchSelect='<select priority='+oObj.priority+' class="hold_location_select" data-hold-id="'+oObj.reserve_id+'" reserve_id="'+oObj.reserve_id+'" name="pick-location" data-pickup-location-source="hold">';
                                 for ( var i=0; i < oObj.branches.length; i++ ){
@@ -225,7 +225,7 @@ $(document).ready(function() {
                     },
                     { "data": { _: "expirationdate_formatted", "sort": "expirationdate" } },
                     {
-                        "mDataProp": function( oObj ) {
+                        "data": function( oObj ) {
                             if ( oObj.priority && parseInt( oObj.priority ) && parseInt( oObj.priority ) > 0 ) {
                                 return oObj.priority;
                             } else {
@@ -234,8 +234,13 @@ $(document).ready(function() {
                         }
                     },
                     {
-                        "bSortable": false,
-                        "mDataProp": function( oObj ) {
+                        "data": function( oObj ) {
+                            return oObj.reservenotes && oObj.reservenotes.escapeHtml() || "";
+                        }
+                    },
+                    {
+                        "orderable": false,
+                        "data": function( oObj ) {
                             return "<select name='rank-request'>"
                                  +"<option value='n'>" + __("No") + "</option>"
                                  +"<option value='del'>" + __("Yes") + "</option>"
@@ -246,9 +251,9 @@ $(document).ready(function() {
                         }
                     },
                     {
-                        "bSortable": false,
+                        "orderable":  false,
                         "visible": SuspendHoldsIntranet,
-                        "mDataProp": function( oObj ) {
+                        "data": function( oObj ) {
                             holds[oObj.reserve_id] = oObj; //Store holds for later use
 
                                 if (oObj.found) {
@@ -339,16 +344,19 @@ $(document).ready(function() {
                         }
                     }
                 ],
-                "bPaginate": false,
-                "bProcessing": true,
-                "bServerSide": false,
+                "paging":  false,
+                "processing":  true,
+                "serverSide":  false,
+                "columnDefs":  [
+                    { "type": "anti-the", "targets": [ "anti-the" ] }
+                ],
                 "ajax": {
                     "url": '/cgi-bin/koha/svc/holds',
                     "data": function ( d ) {
                         d.borrowernumber = borrowernumber;
                     }
                 },
-            }));
+            }, table_settings_holds_table );
 
             $("#holds-table").on("draw.dt", function () {
                 $(".hold-suspend").on("click", function () {
@@ -370,7 +378,7 @@ $(document).ready(function() {
                             alert(__("Unable to resume, hold not found"));
                         }
                         else {
-                            alert(__("Your request could not be processed. Check the logs"));
+                            alert(__("Your request could not be processed. Check the logs for details."));
                         }
                         holdsTable.api().ajax.reload();
                     });
@@ -440,7 +448,7 @@ $(document).ready(function() {
         </div>\
     ");
 
-    $("#suspend-modal-clear-date").on( "click", function() { $("#suspend-modal-until").val(""); } );
+    $("#suspend-modal-clear-date").on( "click", function() { $("#suspend-modal-until").flatpickr().clear(); } );
 
     $("#suspend-modal-submit").on( "click", function( e ) {
         e.preventDefault();
@@ -453,14 +461,14 @@ $(document).ready(function() {
             holdsTable.api().ajax.reload();
         }).error(function (jqXHR, textStatus, errorThrown) {
             if (jqXHR.status === 404) {
-                alert(__("Unable to suspend, hold not found"));
+                alert(__("Unable to suspend, hold not found."));
             }
             else {
-                alert(__("Your request could not be processed. Check the logs"));
+                alert(__("Your request could not be processed. Check the logs for details."));
             }
             holdsTable.api().ajax.reload();
         }).done(function() {
-            $("#suspend-modal-until").val(""); // clean the input
+            $("#suspend-modal-until").flatpickr().clear(); // clean the input
             $('#suspend-modal').modal('hide');
         });
     });
@@ -470,7 +478,32 @@ $(document).ready(function() {
         let reserve_id     = $(this).data('reserve-id');
         let biblionumber   = $(this).data('biblionumber');
         let suspend_until  = $('#suspend_until_' + reserve_id).val();
-        window.location.href='request.pl?action=toggleSuspend&reserve_id=' + reserve_id + '&biblionumber=' + biblionumber + '&suspend_until=' + suspend_until;
+        let suspendForm = $("#hold-actions-form").attr({
+            action: 'request.pl',
+            method: 'post'
+        });
+        let sus_op = $("<input />").attr({
+            type: 'hidden',
+            name: 'op',
+            value: 'cud-toggleSuspend'
+        });
+        let sus_bn = $("<input />").attr({
+            type: 'hidden',
+            name: 'biblionumber',
+            value: biblionumber
+        });
+        let sus_ri = $("<input />").attr({
+            type: 'hidden',
+            name: 'reserve_id',
+            value: reserve_id,
+        });
+        let sus_su = $("<input />").attr({
+            type: 'hidden',
+            name: 'suspend_until',
+            value: suspend_until
+        });
+        suspendForm.append( sus_op, sus_bn, sus_ri, sus_su );
+        $("#hold-actions-form").submit();
         return false;
     });
 });
