@@ -51,7 +51,20 @@ $(document).ready(function() {
         return '';
     }
 
-    // At the moment, the only prefilter possible is borrowernumber
+    function display_request_status(row) {
+        let status       = row._strings.status.str;
+        let status_alias = row._strings.status_av ?
+                                row._strings.status_av.str ?
+                                    row._strings.status_av.str :
+                                row._strings.status_av.code :
+                           null;
+
+        let status_label = status + (status_alias ? " <i><strong>"+status_alias+"</strong></i>" : "");
+
+        return status_label;
+    }
+
+    // Possible prefilters: borrowernumber, batch_id
     // see ill/ill-requests.pl and members/ill-requests.pl
     let additional_prefilters = {};
     if(prefilters){
@@ -63,6 +76,7 @@ $(document).ready(function() {
     }
 
     let borrower_prefilter = additional_prefilters['borrowernumber'] || null;
+    let batch_id_prefilter = additional_prefilters['batch_id'] || null;
 
     let additional_filters = {
         "me.backend": function(){
@@ -78,40 +92,38 @@ $(document).ready(function() {
         "me.borrowernumber": function(){
             return borrower_prefilter ? { "=": borrower_prefilter } : "";
         },
+        "me.batch_id": function(){
+            return batch_id_prefilter ? { "=": batch_id_prefilter } : "";
+        },
         "-or": function(){
             let patron = $("#illfilter_patron").val();
             let status = $("#illfilter_status").val();
+            let status_alias = $("#illfilter_status_alias").val();
             let filters = [];
-            let patron_sub_or = [];
             let status_sub_or = [];
             let subquery_and = [];
 
-            if (!patron && !status) return "";
+            if (!patron && !status && !status_alias) return "";
 
             if(patron){
-                const patron_search_fields = "me.borrowernumber,patron.cardnumber,patron.firstname,patron.surname";
-                patron_search_fields.split(',').forEach(function(attr){
-                    let operator = "=";
-                    let patron_data = patron;
-                    if ( attr != "me.borrowernumber" && attr != "patron.cardnumber") {
-                        operator = "like";
-                        patron_data = "%" + patron + "%";
+                let patronquery = buildPatronSearchQuery(
+                    patron,
+                    {
+                        table_prefix: 'patron',
                     }
-                    patron_sub_or.push({
-                        [attr]:{[operator]: patron_data }
-                    });
-                });
-                subquery_and.push(patron_sub_or);
+                );
+                subquery_and.push(patronquery);
             }
 
             if(status){
-                const status_search_fields = "me.status,me.status_av";
-                status_search_fields.split(',').forEach(function(attr){
-                    status_sub_or.push({
-                        [attr]:{"=": status }
-                    });
-                });
-                subquery_and.push(status_sub_or);
+                subquery_and.push( {"me.status":{"=": status }});
+            }
+            if(status_alias){
+                if (status_alias === 'null' ){
+                    subquery_and.push({ "me.status_alias": { "=": null } });
+                }else{
+                    subquery_and.push({ "me.status_alias": { "=": status_alias } });
+                }
             }
 
             filters.push({"-and": subquery_and});
@@ -119,7 +131,7 @@ $(document).ready(function() {
             return filters;
         },
         "me.placed": function(){
-            if ( Object.keys(additional_prefilters).length ) return "";
+            if (Object.keys(additional_prefilters).length && borrower_prefilter) return "";
             let placed_start = $('#illfilter_dateplaced_start').get(0)._flatpickr.selectedDates[0];
             let placed_end = $('#illfilter_dateplaced_end').get(0)._flatpickr.selectedDates[0];
             if (!placed_start && !placed_end) return "";
@@ -129,7 +141,7 @@ $(document).ready(function() {
             }
         },
         "me.updated": function(){
-            if (Object.keys(additional_prefilters).length) return "";
+            if (Object.keys(additional_prefilters).length && borrower_prefilter) return "";
             let updated_start = $('#illfilter_datemodified_start').get(0)._flatpickr.selectedDates[0];
             let updated_end = $('#illfilter_datemodified_end').get(0)._flatpickr.selectedDates[0];
             if (!updated_start && !updated_end) return "";
@@ -150,7 +162,7 @@ $(document).ready(function() {
             let filters = [];
             let subquery_and = [];
 
-            const search_fields = "me.illrequest_id,me.borrowernumber,me.biblio_id,me.due_date,me.branchcode,library.name,me.status,me.status_alias,me.placed,me.replied,me.updated,me.completed,me.medium,me.accessurl,me.cost,me.price_paid,me.notesopac,me.notesstaff,me.orderid,me.backend,patron.firstname,patron.surname";
+            const search_fields = "me.illrequest_id,me.biblio_id,me.due_date,me.branchcode,library.name,me.status,me.status_alias,me.placed,me.replied,me.updated,me.completed,me.medium,me.accessurl,me.cost,me.price_paid,me.notesopac,me.notesstaff,me.orderid,me.backend";
             let sub_or = [];
             search_fields.split(',').forEach(function(attr){
                 sub_or.push({
@@ -160,6 +172,14 @@ $(document).ready(function() {
             subquery_and.push(sub_or);
             filters.push({"-and": subquery_and});
 
+            let patronquery = buildPatronSearchQuery(
+                keyword,
+                {
+                    table_prefix: 'patron',
+                }
+            );
+            filters.push(patronquery);
+
             const extended_attributes = "title,type,author,article_title,pages,issue,volume,year";
             let extended_sub_or = [];
             subquery_and = [];
@@ -175,8 +195,11 @@ $(document).ready(function() {
     };
 
     let table_id = "#ill-requests";
+
     if (borrower_prefilter) {
         table_id += "-patron-" + borrower_prefilter;
+    } else if ( batch_id_prefilter ){
+        table_id += "-batch-" + batch_id_prefilter;
     }
 
     var ill_requests_table = $(table_id).kohaTable({
@@ -188,6 +211,7 @@ $(document).ready(function() {
             'biblio',
             'comments+count',
             'extended_attributes',
+            'ill_batch',
             'library',
             'id_prefix',
             'patron'
@@ -201,16 +225,33 @@ $(document).ready(function() {
                 "orderable": true,
                 "render": function( data, type, row, meta ) {
                     return '<a href="/cgi-bin/koha/ill/ill-requests.pl?' +
-                            'method=illview&amp;illrequest_id=' +
+                            'op=illview&amp;illrequest_id=' +
                             encodeURIComponent(data) +
                             '">' + escape_str(row.id_prefix) + escape_str(data) + '</a>';
                 }
             },
+            {
+                "data": "ill_batch.name", // batch
+                "orderable": false,
+                "render": function(data, type, row, meta) {
+                    return row.ill_batch ?
+                        '<a href="/cgi-bin/koha/ill/ill-requests.pl?batch_id=' +
+                        row.ill_batch.ill_batch_id +
+                        '">' +
+                        row.ill_batch.name +
+                        '</a>'
+                        : "";
+                }
+            },
             {
                 "data": "", // author
                 "orderable": false,
                 "render": function(data, type, row, meta) {
-                    return display_extended_attribute(row, 'author');
+                    const author = display_extended_attribute(row, 'author');
+                    if(author) return author
+                    const articleAuthor = display_extended_attribute(row, 'article_author');
+                    if(articleAuthor) return articleAuthor
+                    return ''
                 }
             },
             {
@@ -294,12 +335,7 @@ $(document).ready(function() {
                 "data": "status",
                 "orderable": true,
                 "render": function(data, type, row, meta) {
-                    let status_label = row._strings.status_av ?
-                        row._strings.status_av.str ?
-                            row._strings.status_av.str :
-                            row._strings.status_av.code :
-                        row._strings.status.str
-                    return escape_str(status_label);
+                    return display_request_status(row);
                 }
             },
             {
@@ -334,7 +370,8 @@ $(document).ready(function() {
                 "data": "access_url",
                 "orderable": true,
                 "render": function(data, type, row, meta) {
-                    return escape_str(data);
+                    return '<a target="_blank" href="' + data + '">'
+                    + escape_str(data) + '</a>';
                 }
             },
             {
@@ -353,7 +390,7 @@ $(document).ready(function() {
             },
             {
                 "data": "comments_count",
-                "orderable": true,
+                "orderable": false,
                 "searchable": false,
                 "render": function(data, type, row, meta) {
                     return escape_str(data);
@@ -385,11 +422,7 @@ $(document).ready(function() {
                 "orderable": false,
                 "searchable": false,
                 "render": function( data, type, row, meta ) {
-                    return '<a class="btn btn-default btn-sm" ' +
-                            'href="/cgi-bin/koha/ill/ill-requests.pl?' +
-                            'method=illview&amp;illrequest_id=' +
-                            encodeURIComponent(data) +
-                            '">' + ill_manage + '</a>';
+                    return render_table_actions(data);
                 }
             }
         ]
@@ -397,6 +430,20 @@ $(document).ready(function() {
 
     $("#illfilter_form").on('submit', filter);
 
+    function render_table_actions(data) {
+        let actions_string = "";
+        ill_table_actions.forEach((ill_table_action) => {
+            let link_data = ill_table_action.append_column_data_to_link
+                ? encodeURIComponent(data)
+                : "";
+            let link_text = ill_table_action.button_link_translatable_text
+                ? eval(ill_table_action.button_link_translatable_text)
+                : ill_table_action.button_link_text;
+            actions_string += `<a class="${ill_table_action.button_class}" href="${ill_table_action.button_link}${link_data}">${link_text}</a>`;
+        });
+        return actions_string;
+    }
+
     function redrawTable() {
         let table_dt = ill_requests_table.DataTable();
         table_dt.draw();
@@ -418,77 +465,134 @@ $(document).ready(function() {
             $("#"+filter).val("");
         });
 
+        $('#illfilter_status').prop('selectedIndex',0);
+        $('#illfilter_status_alias').prop('selectedIndex',0);
+
         //Clear flatpickr date filters
         $('#illfilter_form > fieldset > ol > li:nth-child(4) > span > a').click();
         $('#illfilter_form > fieldset > ol > li:nth-child(5) > span > a').click();
         $('#illfilter_form > fieldset > ol > li:nth-child(6) > span > a').click();
         $('#illfilter_form > fieldset > ol > li:nth-child(7) > span > a').click();
 
-        disableStatusFilter();
-
         redrawTable();
     }
 
-    function populateStatusFilter(backend) {
-        $.ajax({
-            type: "GET",
-            url: "/api/v1/ill/backends/"+backend,
-            headers: {
-                'x-koha-embed': 'statuses+strings'
-            },
-            success: function(response){
-                let statuses = response.statuses
-                $('#illfilter_status').append(
-                    '<option value="">'+ill_all_statuses+'</option>'
-                );
-                statuses.sort((a, b) => a.str.localeCompare(b.str)).forEach(function(status) {
-                    $('#illfilter_status').append(
-                        '<option value="' + status.code  +
-                        '">' + status.str +  '</option>'
-                    );
-                });
-            }
+    function addStatusOptions(statuses){
+        $('#illfilter_status').children().remove();
+        $('#illfilter_status').append(
+            '<option value="">'+ill_all_statuses+'</option>'
+        );
+        statuses.sort((a, b) => a.str.localeCompare(b.str)).forEach(function(status) {
+            $('#illfilter_status').append(
+                '<option value="' + status.code  +
+                '">' + status.str +  '</option>'
+            );
         });
     }
 
+    function addStatusAliasOptions(status_aliases){
+        $('#illfilter_status_alias').parent().remove();
+        if (status_aliases.length !== 0) {
+            $('#illfilter_status').parent().after(function() {
+            return '<li><label for="illfilter_status_alias">'+ill_status_aliases+':</label> <select name="illfilter_status_alias" id="illfilter_status_alias"></select></li>';
+            });
+            $('#illfilter_status_alias').append(
+                '<option value="">'+ill_all_status_aliases+'</option>'+
+                '<option value="null">'+ill_no_status_alias+'</option>'
+            );
+            status_aliases.sort((a, b) => a.str.localeCompare(b.str)).forEach(function (status_alias) {
+                $('#illfilter_status_alias').append(
+                    '<option value="' + status_alias.code +
+                    '">' + status_alias.str + '</option>'
+                );
+            });
+        }
+    }
+
+    function populateStatusFilter(params) {
+        if(params.backend_statuses){
+            if(params.backend_statuses.statuses){
+                addStatusOptions(params.backend_statuses.statuses);
+            }
+            if(params.backend_statuses.status_aliases){
+                addStatusAliasOptions(params.backend_statuses.status_aliases);
+            }
+        }else if(params.backend){
+            let backend_id = params.backend || "";
+            $.ajax({
+                type: "GET",
+                url: "/api/v1/ill/backends/"+backend_id,
+                headers: {
+                    'x-koha-embed': 'statuses+strings'
+                },
+                success: function(response){
+                    addStatusOptions(response.statuses.filter(status => status.type == 'ill_status'));
+                    addStatusAliasOptions(response.statuses.filter(status => status.type == 'av'));
+                }
+            });
+        }
+    }
+
     function populateBackendFilter() {
         $.ajax({
             type: "GET",
             url: "/api/v1/ill/backends",
+            headers: {
+                'x-koha-embed': 'statuses+strings'
+            },
             success: function(backends){
                 backends.sort((a, b) => a.ill_backend_id.localeCompare(b.ill_backend_id)).forEach(function(backend) {
-                    $('#illfilter_backend').append(
-                        '<option value="' + backend.ill_backend_id  +
-                        '">' + backend.ill_backend_id +  '</option>'
-                    );
+                    if( $('#illfilter_backend option[value=' + backend.ill_backend_id + ']').length == 0 ){
+                        $('#illfilter_backend').append(
+                            '<option value="' + backend.ill_backend_id  +
+                            '">' + backend.ill_backend_id +  '</option>'
+                        );
+                    }
                 });
+
+                let all_existing_statuses = [];
+                backends.forEach((backend) => {
+                    let existing_statuses = backend.statuses;
+                    existing_statuses.filter(status => status.type == 'ill_status').forEach((existing_status) => {
+                        let index = all_existing_statuses.map(function(e) { return e.code; }).indexOf(existing_status.code) || false;
+                        if(index == -1){
+                            all_existing_statuses.push(existing_status);
+                        }
+                    });
+                });
+
+                let all_existing_status_aliases = [];
+                backends.forEach((backend) => {
+                    let existing_status_aliases = backend.statuses;
+                    existing_status_aliases.filter(status => status.type == 'av').forEach((existing_status_aliases) => {
+                        let index = all_existing_status_aliases.map(function(e) { return e.code; }).indexOf(existing_status_aliases.code) || false;
+                        if(index == -1){
+                            all_existing_status_aliases.push(existing_status_aliases);
+                        }
+                    });
+                });
+
+                populateStatusFilter(
+                    { backend_statuses:
+                        {
+                            statuses      : all_existing_statuses,
+                            status_aliases: all_existing_status_aliases
+                        }
+                    }
+                )
             }
         });
     }
 
-    function disableStatusFilter() {
-        $('#illfilter_status').children().remove();
-        $("#illfilter_status").attr('title', ill_manage_select_backend_first);
-        $('#illfilter_status').prop("disabled", true);
-    }
-
-    function enableStatusFilter() {
-        $('#illfilter_status').children().remove();
-        $("#illfilter_status").attr('title', '');
-        $('#illfilter_status').prop("disabled", false);
-    }
-
     $('#illfilter_backend').change(function() {
         var selected_backend = $('#illfilter_backend option:selected').val();
         if (selected_backend && selected_backend.length > 0) {
-            populateStatusFilter(selected_backend);
-            enableStatusFilter();
-        } else {
-            disableStatusFilter();
+            populateStatusFilter( { backend : selected_backend } );
+        } else{
+            populateBackendFilter();
         }
     });
 
-    disableStatusFilter();
     populateBackendFilter();
 
     // Clear all filters
