@@ -37,6 +37,7 @@ use List::MoreUtils qw( uniq );
 use Scalar::Util qw( looks_like_number );
 use Koha::Patron::Attribute::Types;
 use Koha::Patron::Restriction::Types;
+use Koha::Patron::Categories;
 use Koha::Patron::Messages;
 use Koha::CsvProfiles;
 use Koha::Holds;
@@ -62,7 +63,7 @@ my ( $template, $loggedinuser, $cookie ) = get_template_and_user(
         template_name   => $template_name,
         query           => $input,
         type            => "intranet",
-        flagsrequired   => { borrowers => 'edit_borrowers' },
+        flagsrequired   => { borrowers => ['edit_borrowers', 'list_borrowers'] },
     }
 );
 my $borrowernumber = $input->param('borrowernumber');
@@ -91,6 +92,7 @@ if ( $patron->is_debarred ) {
     $template->param(
         'userdebarred'    => $patron->debarred,
         'debarredcomment' => $patron->debarredcomment,
+        'debarredsince'   => $patron->restrictions->search()->single->created,
     );
 
     if ( $patron->debarred ne "9999-12-31" ) {
@@ -119,6 +121,11 @@ $template->param(
 my $relatives_issues_count =
     Koha::Checkouts->count({ borrowernumber => \@relatives });
 
+if ( @guarantees ) {
+    my $total_amount = $patron->relationships_debt({ include_guarantors => 0, only_this_guarantor => 1, include_this_patron => 1 });
+    $template->param( guarantees_fines => $total_amount );
+}
+
 # Calculate and display patron's age
 if ( !$patron->is_valid_age ) {
     $template->param( age_limitations => 1 );
@@ -126,10 +133,10 @@ if ( !$patron->is_valid_age ) {
     $template->param( age_high => $patron->category->upperagelimit );
 }
 
-# Generate CSRF token for upload and delete image buttons
-$template->param(
-    csrf_token => Koha::Token->new->generate_csrf({ session_id => $input->cookie('CGISESSID'),}),
-);
+unless ( Koha::Patron::Categories->search_with_library_limits( { 'me.categorycode' => $patron->categorycode } )->count )
+{
+    $template->param( limited_category => 1 );
+}
 
 if (C4::Context->preference('ExtendedPatronAttributes')) {
     my @attributes = $patron->extended_attributes->as_list; # FIXME Must be improved!
@@ -249,6 +256,19 @@ if( ( my $owing = $account->non_issues_charges ) > 0 ) {
     );
 }
 
+# Check the debt of this patrons guarantors *and* the guarantees of those guarantors
+my $no_issues_charge_guarantors = C4::Context->preference("NoIssuesChargeGuarantorsWithGuarantees");
+if ( $no_issues_charge_guarantors ) {
+    my $guarantors_non_issues_charges = $patron->relationships_debt({ include_guarantors => 1, only_this_guarantor => 0, include_this_patron => 1 });
+
+    if ( $guarantors_non_issues_charges > $no_issues_charge_guarantors ) {
+        $template->param(
+            noissues                      => 1,
+            charges_guarantors_guarantees => $guarantors_non_issues_charges
+        );
+    }
+}
+
 # if the expiry date is before today ie they have expired
 if ( $patron->is_expired ) {
     #borrowercard expired, no issues
@@ -268,6 +288,7 @@ elsif ( $patron->is_going_to_expire ) {
 
 
 my $has_modifications = Koha::Patron::Modifications->search( { borrowernumber => $borrowernumber } )->count;
+my $patron_lists_count = $patron->get_lists_with_patron->count();
 
 $template->param(
     patron          => $patron,
@@ -284,6 +305,7 @@ $template->param(
     logged_in_user => $logged_in_user,
     files => Koha::Patron::Files->new( borrowernumber => $borrowernumber ) ->GetFilesInfo(),
     has_modifications         => $has_modifications,
+    patron_lists_count => $patron_lists_count,
 );
 
 if ( C4::Context->preference('UseRecalls') ) {
