@@ -38,7 +38,7 @@ my $command_line_options = join(" ",@ARGV);
 
 cronlogaction({ info => $command_line_options });
 
-my $recalls = Koha::Recalls->search({ status => 'R' });
+my $recalls = Koha::Recalls->search({ status => 'requested' });
 while( my $recall = $recalls->next ) {
     if ( $recall->should_be_overdue ){
         $recall->set_overdue({ interface => 'COMMANDLINE' });
