@@ -76,6 +76,33 @@ if( !$biblio ) {
     exit;
 }
 
+# If record should be suppressed, handle it early
+if ( C4::Context->preference('OpacSuppression') ) {
+
+    # redirect to opac-blocked info page or 404?
+    my $redirect_url;
+    if ( C4::Context->preference("OpacSuppressionRedirect") ) {
+        $redirect_url = "/cgi-bin/koha/opac-blocked.pl";
+    } else {
+        $redirect_url = "/cgi-bin/koha/errors/404.pl";
+    }
+    if ( $biblio->opac_suppressed() ) {
+
+        # if OPAC suppression by IP address
+        if ( C4::Context->preference('OpacSuppressionByIPRange') ) {
+            my $IPAddress = $ENV{'REMOTE_ADDR'};
+            my $IPRange   = C4::Context->preference('OpacSuppressionByIPRange');
+            if ( $IPAddress !~ /^$IPRange/ ) {
+                print $query->redirect($redirect_url);
+                exit;
+            }
+        } else {
+            print $query->redirect($redirect_url);
+            exit;
+        }
+    }
+}
+
 #open template
 my ( $template, $loggedinuser, $cookie ) = get_template_and_user(
     {
@@ -168,29 +195,30 @@ $template->param(
     subscriptionsnumber => $subscriptionsnumber,
 );
 
-my $can_item_be_reserved = 0;
 my $res = GetISBDView({
     'record'    => $record,
     'template'  => 'opac',
     'framework' => $biblio->frameworkcode
 });
 
-my $items = $biblio->items;
-while ( my $item = $items->next ) {
+# Count the number of items that allow holds at the 'All libraries' rule level
+my $holdable_items = $biblio->items->filter_by_for_hold->count;
 
-    $can_item_be_reserved = $can_item_be_reserved || $patron && IsAvailableForItemLevelRequest( $item, $patron, undef );
+# If we have a patron we need to check their policies for holds in the loop below
+# If we don't have a patron, then holdable items determines holdability
+my $can_holds_be_placed = $patron ? 0 : $holdable_items;
+if ($patron) {
+    my $items = $biblio->items;
+    while ( my $item = $items->next ) {
+        $can_holds_be_placed = $can_holds_be_placed || IsAvailableForItemLevelRequest( $item, $patron, undef );
+    }
 }
 
-if( $can_item_be_reserved || CountItemsIssued($biblionumber) || $biblio->has_items_waiting_or_intransit ) {
-    $template->param( ReservableItems => 1 );
-}
-
-my $norequests = ! $items->filter_by_for_hold->count;
-
 $template->param(
-    norequests   => $norequests,
-    ISBD         => $res,
-    biblio       => $biblio,
+    ReservableItems => $can_holds_be_placed,
+    ISBD            => $res,
+    biblio          => $biblio,
+    borrowernumber  => $loggedinuser,
 );
 
 #Search for title in links
